from resources.lib.gui.renderers.dialog_renderer import DialogRenderer
from resources.lib.utils.kodiutils import get_string, notification, show_settings
from resources.lib.storage.settings import settings
from resources.lib.kodilogging import logger
from resources.lib.const import SETTINGS, MEDIA_TYPE, URL, ENDPOINT, ROUTE
import xbmc
import xbmcvfs
from os.path import join
import urllib2
from urllib2 import HTTPError
import json

def normalizeName(name):
    name = name.replace(': ', ' ')
    name = name.replace(':', ' ')
    name = name.replace('*', '.')
    return name

def getMediaDetailURL(id):
    return URL.API + ENDPOINT.MEDIA + "/" + id

def writeNFOFile(name, id, type):
    url = getMediaDetailURL(id)
    detailContents = ''
    try:
        detailContents = urllib2.urlopen(url).read()
    except HTTPError as err:
        logger.error(err)
    if(detailContents != ''):
        mediaDetail = json.loads(detailContents)
        if('services' in mediaDetail):
            services = mediaDetail['services']
            file = xbmcvfs.File(name, 'w')
            if('csfd' in services):
                csfdId = services['csfd'] 
                file.write("https://www.csfd.cz/film/{}".format(csfdId) + "\n")
            if('tmdb' in services):
                tmdbId = services['tmdb']        
                file.write("https://www.themoviedb.org/{}/{}".format(type, tmdbId) + "\n")
            if('imdb' in services):
                imdbId = services['imdb'] 
                file.write("https://www.imdb.com/title/{}".format(imdbId) + "\n")
            file.close()
 
def add_movie(mf):
    logger.debug("Adding movie to library... BEGIN")
    originalTitle   = xbmc.getInfoLabel("ListItem.OriginalTitle").decode('utf-8')
    originalTitle   = normalizeName(originalTitle)
    title           = xbmc.getInfoLabel("ListItem.Title").decode('utf-8')
    title           = normalizeName(title)
    year            = xbmc.getInfoLabel("ListItem.Year")
    moviePath       = xbmc.getInfoLabel("ListItem.FileNameAndPath")
    mediaId         = moviePath[-24:]

    if(originalTitle == ''):
        originalTitle = title
    dirName = mf + originalTitle + ' (' + year + ')'
    
    if not xbmcvfs.exists(dirName):
        xbmcvfs.mkdir(dirName)
    strmFileName = join(dirName, originalTitle + ' (' + year + ')' + '.strm')
    nfoFileName  = join(dirName, originalTitle + ' (' + year + ')' + '.nfo') 

    writeNFOFile(nfoFileName, mediaId, 'movie')    

    file = xbmcvfs.File(strmFileName, 'w')
    file.write(str(moviePath))
    file.close()
    # eventualne hodit notifikaciu
    # notification(get_string(30405), get_string(30406) % originalTitle)
    logger.debug("Adding movie to library... END")
    xbmc.executebuiltin('UpdateLibrary(video)')

def add_tvshow(tf):
    logger.debug("Adding TV show to library... BEGIN")
    dp = DialogRenderer.progress(get_string(30351), get_string(30352))

    originalTitle   = xbmc.getInfoLabel("ListItem.Originaltitle").decode('utf-8')
    originalTitle   = normalizeName(originalTitle)
    year            = xbmc.getInfoLabel("ListItem.Year")

    dirName = tf + originalTitle + ' (' + year + ')'

    if not xbmcvfs.exists(dirName):
        xbmcvfs.mkdir(dirName)

    # get media ID of TV Show
    tvshowPath = xbmc.getInfoLabel("ListItem.FileNameAndPath")
    mediaId = tvshowPath[-24:]

    mediaURL = URL.API + ENDPOINT.FILTER
    # "https://plugin.sc2.zone/api/media/filter/parent?value={}&sort=episode"
    streamURL = ROUTE.PROCESS_MEDIA_ITEM + "?url=%2Fapi%2Fmedia%2F{}%2Fstreams&media_id={}&root_parent_id={}" 
    # "plugin://plugin.video.stream-cinema-2-release/process_media_item/?url=%2Fapi%2Fmedia%2F{}%2Fstreams&media_id={}&root_parent_id={}"

    nfoFileName  = join(dirName, originalTitle + ' (' + year + ')' + '.nfo') 
    writeNFOFile(nfoFileName, mediaId, 'tv')    

    url = mediaURL.format(filter_name = "parent", query = "value=" + mediaId + "&sort=episode")
    # url = mediaURL.format(mediaId)
    tvShowContents = ''
    try:
        tvShowContents = urllib2.urlopen(url).read()
    except HTTPError as err:
        logger.error(err)
    if(tvShowContents != ''):
        seasonData = json.loads(tvShowContents)
        seasonsCount = seasonData['totalCount']
        s = 1;
        seasons = seasonData['data']
        for season in seasons:
            episodeNo = season['_source']['info_labels']['episode']
            
            if(episodeNo != 0):
                seasonDirName = join(dirName, "Season 01")
                if not xbmcvfs.exists(seasonDirName):
                    xbmcvfs.mkdir(seasonDirName)
                episodeId = season['_id']
                parentId = season['_source']['root_parent']
                episodeURL = streamURL.format(episodeId, episodeId, parentId)
                episodeFileName = "S01E{}.strm".format(str(episodeNo).zfill(2))
                file = xbmcvfs.File(join(seasonDirName, episodeFileName), 'w')
                file.write(str(episodeURL))
                file.close()
            
            else:        
                currentSeason = "Season {}".format(str(s).zfill(2))
                seasonId = season["_id"]
                seasonDirName = join(dirName, currentSeason)
                if not xbmcvfs.exists(seasonDirName):
                    xbmcvfs.mkdir(seasonDirName)
                
                url = mediaURL.format(filter_name = "parent", query = "value=" + seasonId + "&sort=episode")
                # url = mediaURL.format(seasonId)
                episodesContents = ''
                try:
                    episodesContents = urllib2.urlopen(url).read()
                except HTTPError as err:
                    logger.error(err)
                if(episodesContents != ''):
                    episodesData = json.loads(episodesContents) 
                    episodes = episodesData['data']
                    episodesCount = episodesData['totalCount']
                    e = 1
                    for episode in episodes:
                        seasonNo = episode['_source']['info_labels']['season']
                        episodeNo = episode['_source']['info_labels']['episode']
                        episodeFileName = "S{}E{}.strm".format(str(seasonNo).zfill(2), str(episodeNo).zfill(2))
                        episodeId = episode['_id']
                        episodeURL = streamURL.format(episodeId, episodeId, seasonId)
                        file = xbmcvfs.File(join(seasonDirName, episodeFileName), 'w')
                        file.write(str(episodeURL))
                        file.close()
                        e = e+1
            s = s+1
            dp.update(int(100*float(s)/float(len(seasons))))

    dp.close()
    logger.debug("Adding TV show to library... END")
    xbmc.executebuiltin('UpdateLibrary(video)')

def add_to_library():
    dbtype  = xbmc.getInfoLabel("ListItem.dbtype")
    if dbtype == MEDIA_TYPE.MOVIE:
        mf = settings[SETTINGS.MOVIE_LIBRARY_FOLDER]
        add_movie(mf)

    if (dbtype == MEDIA_TYPE.TV_SHOW):
        tf = settings[SETTINGS.TV_SHOW_LIBRARY_FOLDER]
        add_tvshow(tf)
